@@ -11,10 +11,12 @@ no warnings 'uninitialized';
 use Carp;
 use List::Util   qw/max/;
 use Exporter     qw/import/;
+use Scalar::Does qw/does/;
 use DBI;
 use Try::Tiny;
 use Module::Load ();
 
+
 {no strict 'refs'; *CARP_NOT = \@DBIx::DataModel::CARP_NOT;}
 
 our @EXPORT = qw/fromDBIxClass fromDBI/;
@@ -81,7 +83,7 @@ sub parse_DBI {
 
   # dbh connection
   my $arg1    = shift or croak "missing arg (dsn for DBI->connect(..))";
-  my $dbh = (ref $arg1 && $arg1->isa('DBI::db')) ? $arg1 : do {
+  my $dbh = does($arg1, 'DBI::db') ? $arg1 : do {
     my $user    = shift || "";
     my $passwd  = shift || "";
     my $options = shift || {RaiseError => 1};
@@ -548,7 +550,7 @@ subclass that will be generated (default is C<My::Schema>).
 
   $generator->fromDBI(@dbi_connection_args, $catalog, $schema, $type);
   # or
-  fromDBI(@dbi_connection_args, $catalog, $schema);
+  fromDBI(@dbi_connection_args, $catalog, $schema, $type);
 
 Connects to a L<DBI|DBI> data source, gathers information from the
 database about tables, primary and foreign keys, and generates
